@@ -1,5 +1,4 @@
 import argparse
-import json
 import logging
 import sys
 import uuid
@@ -14,6 +13,7 @@ from annofabapi.models import (
     AdditionalDataDefinitionType,
     AdditionalDataDefinitionV1,
     AnnotationDataHoldingType,
+    AnnotationType,
     LabelV1,
     ProjectMemberRole,
     TaskStatus,
@@ -24,7 +24,7 @@ from annofabapi.parser import (
     lazy_parse_simple_annotation_dir_by_task,
     lazy_parse_simple_annotation_zip_by_task,
 )
-from annofabapi.utils import can_put_annotation
+from annofabapi.utils import can_put_annotation, str_now
 from dataclasses_json import DataClassJsonMixin
 from more_itertools import first_true
 
@@ -45,15 +45,15 @@ class ImportedSimpleAnnotationDetail(DataClassJsonMixin):
     label: str
     """アノテーション仕様のラベル名(英語)"""
 
-    annotation_id: Optional[str]
-    """アノテーションID"""
-
     data: Dict[str, Any]
     """"""
 
     attributes: Dict[str, Union[str, bool, int]]
     """属性情報。キーは属性の名前、値は属性の値。 """
 
+    annotation_id: Optional[str] = None
+    """アノテーションID"""
+
 
 @dataclass
 class ImportedSimpleAnnotation(DataClassJsonMixin):
@@ -109,19 +109,12 @@ class ImportAnnotation(AbstractCommandLineInterface):
         else:
             return AnnotationDataHoldingType.INNER
 
-    @staticmethod
-    def _create_annotation_id(data: FullAnnotationData, label_id: str) -> str:
-        if data["_type"] == "Classification":
-            return label_id
-        else:
-            return str(uuid.uuid4())
-
     def _to_additional_data_list(self, attributes: Dict[str, Any], label_info: LabelV1) -> List[AdditionalData]:
         additional_data_list: List[AdditionalData] = []
         for key, value in attributes.items():
             specs_additional_data = self._get_additional_data_from_attribute_name(key, label_info)
             if specs_additional_data is None:
-                logger.warning(f"attribute_name={key} が存在しません。")
+                logger.warning(f"アノテーション仕様に attribute_name={key} が存在しません。")
                 continue
 
             additional_data = AdditionalData(
@@ -154,7 +147,7 @@ class ImportAnnotation(AbstractCommandLineInterface):
         return additional_data_list
 
     def _to_annotation_detail_for_request(
-        self, project_id: str, parser: SimpleAnnotationParser, detail: ImportedSimpleAnnotationDetail
+        self, project_id: str, parser: SimpleAnnotationParser, detail: ImportedSimpleAnnotationDetail, now_datetime: str
     ) -> Optional[AnnotationDetail]:
         """
         Request Bodyに渡すDataClassに変換する。塗りつぶし画像があれば、それをS3にアップロードする。
@@ -165,18 +158,29 @@ class ImportAnnotation(AbstractCommandLineInterface):
             detail:
 
         Returns:
+            変換できない場合はNoneを返す
 
         """
         label_info = self.get_label_info_from_label_name(detail.label)
         if label_info is None:
             return None
 
+        def _get_annotation_id(arg_label_info: LabelV1) -> str:
+            if detail.annotation_id is not None:
+                return detail.annotation_id
+            else:
+                if arg_label_info["annotation_type"] == AnnotationType.CLASSIFICATION.value:
+                    # 全体アノテーションの場合、annotation_idはlabel_idである必要がある
+                    return arg_label_info["label_id"]
+                else:
+                    return str(uuid.uuid4())
+
         additional_data_list: List[AdditionalData] = self._to_additional_data_list(detail.attributes, label_info)
         data_holding_type = self._get_data_holding_type_from_data(detail.data)
 
         dest_obj = AnnotationDetail(
             label_id=label_info["label_id"],
-            annotation_id=detail.annotation_id if detail.annotation_id is not None else str(uuid.uuid4()),
+            annotation_id=_get_annotation_id(label_info),
             account_id=self.service.api.account_id,
             data_holding_type=data_holding_type,
             data=detail.data,
@@ -185,8 +189,8 @@ class ImportAnnotation(AbstractCommandLineInterface):
             etag=None,
             url=None,
             path=None,
-            created_datetime=None,
-            updated_datetime=None,
+            created_datetime=now_datetime,
+            updated_datetime=now_datetime,
         )
 
         if data_holding_type == AnnotationDataHoldingType.OUTER:
@@ -206,12 +210,14 @@ class ImportAnnotation(AbstractCommandLineInterface):
         old_annotation: Optional[Dict[str, Any]] = None,
     ) -> Dict[str, Any]:
         request_details: List[Dict[str, Any]] = []
+        now_datetime = str_now()
         for detail in details:
-            request_detail = self._to_annotation_detail_for_request(project_id, parser, detail)
+            request_detail = self._to_annotation_detail_for_request(
+                project_id, parser, detail, now_datetime=now_datetime
+            )
 
             if request_detail is not None:
-                # Enumをシリアライズするため、一度JSONにしてからDictに変換する
-                request_details.append(json.loads(request_detail.to_json()))
+                request_details.append(request_detail.to_dict(encode_json=True))
 
         updated_datetime = old_annotation["updated_datetime"] if old_annotation is not None else None
 
@@ -249,6 +255,7 @@ class ImportAnnotation(AbstractCommandLineInterface):
             logger.debug(
                 f"task_id={task_id}, input_data_id={input_data_id} : "
                 f"インポート先のタスクに既にアノテーションが存在するため、アノテーションの登録をスキップします。"
+                f"アノテーションを上書きする場合は、`--overwrite` を指定してください。"
             )
             return False
 
@@ -273,14 +280,15 @@ class ImportAnnotation(AbstractCommandLineInterface):
                     success_count += 1
             except Exception as e:  # pylint: disable=broad-except
                 logger.warning(
-                    f"task_id={parser.task_id}, input_data_id={parser.input_data_id} のアノテーションインポートに失敗しました。: {e}"
+                    f"task_id={parser.task_id}, input_data_id={parser.input_data_id} の"
+                    f"アノテーションインポートに失敗しました。: {type(e).__name__}: {e}"
                 )
 
         logger.info(f"タスク'{task_parser.task_id}'の入力データ {success_count} 個に対してアノテーションをインポートしました。")
         return success_count
 
     def execute_task(
-        self, project_id: str, task_parser: SimpleAnnotationParserByTask, my_account_id: str, overwrite: bool = False
+        self, project_id: str, task_parser: SimpleAnnotationParserByTask, overwrite: bool, force: bool
     ) -> bool:
         """
         1個のタスクに対してアノテーションを登録する。
@@ -288,7 +296,6 @@ class ImportAnnotation(AbstractCommandLineInterface):
         Args:
             project_id:
             task_parser:
-            my_account_id: 自分自身のアカウントID
             overwrite:
 
         Returns:
@@ -310,11 +317,31 @@ class ImportAnnotation(AbstractCommandLineInterface):
             logger.info(f"タスク'{task_id}'は作業中または受入完了状態のため、インポートをスキップします。 status={task['status']}")
             return False
 
-        if not can_put_annotation(task, my_account_id):
-            logger.debug(f"タスク'{task_id}'は、過去に誰かに割り当てられたタスクで、現在の担当者が自分自身でないため、アノテーションのインポートをスキップします。")
-            return False
+        old_account_id: Optional[str] = None
+        changed_operator = False
+        if force:
+            if not can_put_annotation(task, self.service.api.account_id):
+                logger.debug(f"タスク'{task_id}' の担当者を自分自身に変更します。")
+                self.service.wrapper.change_task_operator(
+                    project_id, task_id, operator_account_id=self.service.api.account_id
+                )
+                changed_operator = True
+                old_account_id = task["account_id"]
+
+        else:
+            if not can_put_annotation(task, self.service.api.account_id):
+                logger.debug(
+                    f"タスク'{task_id}'は、過去に誰かに割り当てられたタスクで、現在の担当者が自分自身でないため、アノテーションのインポートをスキップします。"
+                    f"担当者を自分自身に変更してアノテーションを登録する場合は `--force` を指定してください。"
+                )
+                return False
 
         result_count = self.put_annotation_for_task(project_id, task_parser, overwrite)
+        if changed_operator:
+            logger.debug(f"タスク'{task_id}' の担当者を元に戻します。")
+            old_account_id = task["account_id"]
+            self.service.wrapper.change_task_operator(project_id, task_id, operator_account_id=old_account_id)
+
         return result_count > 0
 
     @staticmethod
@@ -345,7 +372,6 @@ class ImportAnnotation(AbstractCommandLineInterface):
         super().validate_project(project_id, [ProjectMemberRole.OWNER])
 
         task_id_list = annofabcli.common.cli.get_list_from_args(args.task_id)
-        my_account_id = self.service.api.account_id
 
         # Simpleアノテーションの読み込み
         if annotation_path.is_file():
@@ -359,19 +385,15 @@ class ImportAnnotation(AbstractCommandLineInterface):
                 if len(task_id_list) > 0:
                     # コマンドライン引数で --task_idが指定された場合は、対象のタスクのみインポートする
                     if task_parser.task_id in task_id_list:
-                        if self.execute_task(
-                            project_id, task_parser, overwrite=args.overwrite, my_account_id=my_account_id
-                        ):
+                        if self.execute_task(project_id, task_parser, overwrite=args.overwrite, force=args.force):
                             success_count += 1
                 else:
                     # コマンドライン引数で --task_idが指定されていない場合はすべてをインポートする
-                    if self.execute_task(
-                        project_id, task_parser, overwrite=args.overwrite, my_account_id=my_account_id
-                    ):
+                    if self.execute_task(project_id, task_parser, overwrite=args.overwrite, force=args.force):
                         success_count += 1
 
             except Exception as e:  # pylint: disable=broad-except
-                logger.warning(f"task_id={task_parser.task_id} のアノテーションインポートに失敗しました。: {e}")
+                logger.warning(f"task_id={task_parser.task_id} のアノテーションインポートに失敗しました。: {type(e).__name__}: {e}")
 
         logger.info(f"{success_count} 個のタスクに対してアノテーションをインポートしました。")
 
@@ -402,6 +424,10 @@ def parse_args(parser: argparse.ArgumentParser):
         help="指定した場合、すでに存在するアノテーションを上書きします（入力データ単位）。" "指定しなければ、アノテーションのインポートをスキップします。",
     )
 
+    parser.add_argument(
+        "--force", action="store_true", help="過去に割り当てられていて現在の担当者が自分自身でない場合、タスクの担当者を自分自身に変更してからアノテーションをインポートします。"
+    )
+
     parser.set_defaults(subcommand_func=main)
 
 
@@ -409,9 +435,7 @@ def add_parser(subparsers: argparse._SubParsersAction):
     subcommand_name = "import"
     subcommand_help = "アノテーションをインポートします。"
     description = (
-        "アノテーションをインポートします。"
-        "アノテーションのフォーマットは、Simpleアノテーション(v2)と同じフォルダ構成のzipファイルまたはディレクトリです。"
-        "ただし、作業中/完了状態のタスク、または「過去に割り当てられていて現在の担当者が自分自身でない」タスクはリストアできません。"
+        "アノテーションをインポートします。アノテーションのフォーマットは、Simpleアノテーションと同じフォルダ構成のzipファイルまたはディレクトリです。ただし、作業中/完了状態のタスクはインポートできません。"
     )
     epilog = "チェッカーまたはオーナロールを持つユーザで実行してください。"
 
