@@ -20,6 +20,8 @@ from annofabapi.models import (
 )
 from dataclasses_json import DataClassJsonMixin
 
+from annofabcli.common.exceptions import OrganizationAuthorizationError, ProjectAuthorizationError
+
 logger = logging.getLogger(__name__)
 
 
@@ -72,6 +74,7 @@ class TaskQuery(DataClassJsonMixin):
     コマンドライン上で指定するタスクの検索条件
     """
 
+    task_id: Optional[str] = None
     phase: Optional[TaskPhase] = None
     status: Optional[TaskStatus] = None
     phase_stage: Optional[int] = None
@@ -106,9 +109,19 @@ def match_task_with_query(  # pylint: disable=too-many-return-statements
     Returns:
         trueならタスククエリ条件に合致する。
     """
+
+    def match_str(name: str, query: str, ignore_case: bool) -> bool:
+        if ignore_case:
+            return query.lower() in name.lower()
+        else:
+            return query in name
+
     if task_query is None:
         return True
 
+    if task_query.task_id is not None and not match_str(task.task_id, task_query.task_id, ignore_case=True):
+        return False
+
     if task_query.status is not None and task.status != task_query.status:
         return False
 
@@ -228,16 +241,6 @@ class AnnofabApiFacade:
         project, _ = self.service.api.get_project(project_id)
         return project["title"]
 
-    def get_my_account_id(self) -> str:
-        """
-        自分自身のaccount_idを取得する
-        Returns:
-            account_id
-
-        """
-        account, _ = self.service.api.get_my_account()
-        return account["account_id"]
-
     def _get_organization_member_with_predicate(
         self, project_id: str, predicate: Callable[[Any], bool]
     ) -> Optional[OrganizationMember]:
@@ -313,7 +316,7 @@ class AnnofabApiFacade:
 
         Args:
             project_id:
-            accoaunt_id:
+            account_id:
 
         Returns:
             プロジェクトメンバ。見つからない場合はNone
@@ -837,3 +840,33 @@ class AnnofabApiFacade:
         if task_query.user_id is not None:
             task_query.account_id = self.get_account_id_from_user_id(project_id, task_query.user_id)
         return task_query
+
+    def validate_project(
+        self,
+        project_id,
+        project_member_roles: Optional[List[ProjectMemberRole]] = None,
+        organization_member_roles: Optional[List[OrganizationMemberRole]] = None,
+    ):
+        """
+        プロジェクト or 組織に対して、必要な権限が付与されているかを確認する。
+
+        Args:
+            project_id:
+            project_member_roles: プロジェクトメンバロールの一覧. Noneの場合はチェックしない。
+            organization_member_roles: 組織メンバロールの一覧。Noneの場合はチェックしない。
+
+        Raises:
+             AuthorizationError: 自分自身のRoleがいずれかのRoleにも合致しなければ、AuthorizationErrorが発生する。
+
+        """
+        project_title = self.get_project_title(project_id)
+        logger.info(f"project_title = {project_title}, project_id = {project_id}")
+
+        if project_member_roles is not None:
+            if not self.contains_any_project_member_role(project_id, project_member_roles):
+                raise ProjectAuthorizationError(project_title, project_member_roles)
+
+        if organization_member_roles is not None:
+            organization_name = self.get_organization_name_from_project_id(project_id)
+            if not self.contains_any_organization_member_role(organization_name, organization_member_roles):
+                raise OrganizationAuthorizationError(organization_name, organization_member_roles)
