@@ -1,13 +1,13 @@
 import argparse
 import importlib
-import json
 import logging.handlers
 import re
+import sys
 from dataclasses import dataclass
 from functools import partial
 from multiprocessing import Pool
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Callable, List, Optional
 
 import annofabapi
 import pandas
@@ -17,10 +17,11 @@ from dataclasses_json import DataClassJsonMixin
 import annofabcli
 from annofabcli import AnnofabApiFacade
 from annofabcli.common.cli import AbstractCommandLineInterface, build_annofabapi_resource_and_login
+from annofabcli.common.facade import TaskQuery
+from annofabcli.stat_visualization.merge_visualization_dir import merge_visualization_dir
 from annofabcli.statistics.csv import FILENAME_WHOLE_PEFORMANCE, Csv, write_summarise_whole_peformance_csv
 from annofabcli.statistics.database import Database, Query
 from annofabcli.statistics.linegraph import LineGraph, OutputTarget
-from annofabcli.statistics.merge_visualization_dir import merge_visualization_dir
 from annofabcli.statistics.scatter import Scatter
 from annofabcli.statistics.table import AggregationBy, Table
 
@@ -29,7 +30,7 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class CommnadLineArgs(DataClassJsonMixin):
-    task_query: Optional[Dict[str, Any]]
+    task_query: TaskQuery
     user_id_list: Optional[List[str]]
     start_date: Optional[str]
     end_date: Optional[str]
@@ -58,15 +59,16 @@ def write_project_name_file(
     filename = annofabcli.utils.to_filename(project_title)
     output_project_dir.mkdir(exist_ok=True, parents=True)
 
-    prject_summary = ProjectSummary(
+    project_summary = ProjectSummary(
         project_id=project_id,
         project_title=project_title,
         measurement_datetime=annofabapi.utils.str_now(),
         args=command_line_args,
     )
 
+    print(project_summary)
     with open(str(output_project_dir / f"{filename}.json"), "w") as f:
-        json.dump(prject_summary.to_dict(), f, ensure_ascii=False, indent=2)
+        f.write(project_summary.to_json(ensure_ascii=False, indent=2))
 
 
 def catch_exception(function: Callable[..., Any]) -> Callable[..., Any]:
@@ -388,7 +390,8 @@ def visualize_statistics(
     annofab_facade: AnnofabApiFacade,
     work_dir: Path,
     output_project_dir: Path,
-    task_query: Dict[str, Any],
+    task_query: TaskQuery,
+    task_id_list: Optional[List[str]],
     ignored_task_id_list: Optional[List[str]],
     user_id_list: Optional[List[str]],
     update: bool = False,
@@ -418,8 +421,9 @@ def visualize_statistics(
         project_id,
         str(checkpoint_dir),
         query=Query(
-            task_query_param=task_query,
-            ignored_task_id_list=ignored_task_id_list,
+            task_query=task_query,
+            task_id_set=set(task_id_list) if task_id_list is not None else None,
+            ignored_task_id_set=set(ignored_task_id_list) if ignored_task_id_list is not None else None,
             start_date=start_date,
             end_date=end_date,
         ),
@@ -483,7 +487,8 @@ def visualize_statistics_wrapper(
     annofab_service: annofabapi.Resource,
     annofab_facade: AnnofabApiFacade,
     work_dir: Path,
-    task_query: Dict[str, Any],
+    task_query: TaskQuery,
+    task_id_list: Optional[List[str]],
     ignored_task_id_list: Optional[List[str]],
     user_id_list: Optional[List[str]],
     update: bool = False,
@@ -504,6 +509,7 @@ def visualize_statistics_wrapper(
             work_dir=work_dir,
             output_project_dir=output_project_dir,
             task_query=task_query,
+            task_id_list=task_id_list,
             ignored_task_id_list=ignored_task_id_list,
             user_id_list=user_id_list,
             update=update,
@@ -528,7 +534,8 @@ class VisualizeStatistics(AbstractCommandLineInterface):
         root_output_dir: Path,
         project_id_list: List[str],
         work_dir: Path,
-        task_query: Dict[str, Any],
+        task_query: TaskQuery,
+        task_id_list: Optional[List[str]],
         ignored_task_id_list: Optional[List[str]],
         user_id_list: Optional[List[str]],
         update: bool = False,
@@ -548,6 +555,7 @@ class VisualizeStatistics(AbstractCommandLineInterface):
                 annofab_facade=self.facade,
                 work_dir=work_dir,
                 task_query=task_query,
+                task_id_list=task_id_list,
                 ignored_task_id_list=ignored_task_id_list,
                 user_id_list=user_id_list,
                 update=update,
@@ -570,6 +578,7 @@ class VisualizeStatistics(AbstractCommandLineInterface):
                     project_id=project_id,
                     work_dir=work_dir,
                     task_query=task_query,
+                    task_id_list=task_id_list,
                     ignored_task_id_list=ignored_task_id_list,
                     user_id_list=user_id_list,
                     update=update,
@@ -583,12 +592,32 @@ class VisualizeStatistics(AbstractCommandLineInterface):
 
         return output_project_dir_list
 
+    @staticmethod
+    def validate(args: argparse.Namespace) -> bool:
+        COMMON_MESSAGE = "annofabcli statistics visulize: error:"
+        if args.start_date is not None and args.end_date is not None:
+            if args.start_date > args.end_date:
+                print(
+                    f"{COMMON_MESSAGE} argument `START_DATE <= END_DATE` の関係を満たしていません。",
+                    file=sys.stderr,
+                )
+                return False
+
+        return True
+
     def main(self):
         args = self.args
-        task_query = annofabcli.common.cli.get_json_from_args(args.task_query)
+        if not self.validate(args):
+            return
+
+        dict_task_query = annofabcli.common.cli.get_json_from_args(args.task_query)
+        task_query: Optional[TaskQuery] = TaskQuery.from_dict(dict_task_query) if dict_task_query is not None else None
+
         ignored_task_id_list = (
             annofabcli.common.cli.get_list_from_args(args.ignored_task_id) if args.ignored_task_id is not None else None
         )
+        task_id_list = annofabcli.common.cli.get_list_from_args(args.task_id) if args.task_id is not None else None
+
         user_id_list = annofabcli.common.cli.get_list_from_args(args.user_id) if args.user_id is not None else None
         project_id_list = annofabcli.common.cli.get_list_from_args(args.project_id)
 
@@ -607,10 +636,11 @@ class VisualizeStatistics(AbstractCommandLineInterface):
                 output_project_dir=root_output_dir,
                 work_dir=work_dir,
                 task_query=task_query,
+                task_id_list=task_id_list,
                 ignored_task_id_list=ignored_task_id_list,
                 user_id_list=user_id_list,
                 update=not args.not_update,
-                download_latest=args.download_latest,
+                download_latest=args.latest,
                 start_date=args.start_date,
                 end_date=args.end_date,
                 minimal_output=args.minimal,
@@ -623,10 +653,11 @@ class VisualizeStatistics(AbstractCommandLineInterface):
                 project_id_list=project_id_list,
                 work_dir=work_dir,
                 task_query=task_query,
+                task_id_list=task_id_list,
                 ignored_task_id_list=ignored_task_id_list,
                 user_id_list=user_id_list,
                 update=not args.not_update,
-                download_latest=args.download_latest,
+                download_latest=args.latest,
                 start_date=args.start_date,
                 end_date=args.end_date,
                 minimal_output=args.minimal,
@@ -650,7 +681,7 @@ class VisualizeStatistics(AbstractCommandLineInterface):
                     csv_path_list=whole_peformance_csv_list, output_path=root_output_dir / "プロジェクトごとの生産性と品質.csv"
                 )
             else:
-                logger.warning(f"出力した統計情報は0なので、`プロジェクトごとの生産性と品質.csv`を出力しません。")
+                logger.warning(f"出力した統計情報は0件なので、`プロジェクトごとの生産性と品質.csv`を出力しません。")
 
 
 def main(args):
@@ -687,7 +718,16 @@ def parse_args(parser: argparse.ArgumentParser):
         type=str,
         help="タスクの検索クエリをJSON形式で指定します。指定しない場合はすべてのタスクを取得します。"
         "`file://`を先頭に付けると、JSON形式のファイルを指定できます。"
-        "クエリのキーは、phase, status, task_id のみです。[getTasks API](https://annofab.com/docs/api/#operation/getTasks) 参照",
+        "クエリのキーは、task_id, phase, phase_stage, status のみです。",
+    )
+
+    parser.add_argument(
+        "-t",
+        "--task_id",
+        type=str,
+        required=False,
+        nargs="+",
+        help="集計対象のタスクのtask_idを指定します。" + "`file://`を先頭に付けると、task_idの一覧が記載されたファイルを指定できます。",
     )
 
     parser.add_argument("--start_date", type=str, help="指定した日付（'YYYY-MM-DD'）以降に教師付を開始したタスクを集計する。")
@@ -696,7 +736,7 @@ def parse_args(parser: argparse.ArgumentParser):
     parser.add_argument(
         "--ignored_task_id",
         nargs="+",
-        help=("可視化対象外のタスクのtask_id。" "指定しない場合は、すべてのタスクが可視化対象です。" "file://`を先頭に付けると、一覧が記載されたファイルを指定できます。"),
+        help=("集計対象外のタスクのtask_idを指定します。`--task_id` より優先度が高いです。" "file://`を先頭に付けると、一覧が記載されたファイルを指定できます。"),
     )
 
     parser.add_argument(
@@ -706,7 +746,7 @@ def parse_args(parser: argparse.ArgumentParser):
     )
 
     parser.add_argument(
-        "--download_latest",
+        "--latest",
         action="store_true",
         help="統計情報の元になるファイル（アノテーションzipなど）の最新版をダウンロードします。ファイルを最新版にするのに5分以上待つ必要があります。",
     )
@@ -740,8 +780,8 @@ def parse_args(parser: argparse.ArgumentParser):
 
 def add_parser(subparsers: argparse._SubParsersAction):
     subcommand_name = "visualize"
-    subcommand_help = "統計情報を可視化したファイルを出力します。"
-    description = "統計情報を可視化したファイルを出力します。毎日 03:00JST頃に更新されます。"
+    subcommand_help = "生産性に関するCSVファイルやグラフを出力します。"
+    description = "生産性に関するCSVファイルやグラフを出力します。"
     epilog = "アノテーションユーザまたはオーナロールを持つユーザで実行してください。"
 
     parser = annofabcli.common.cli.add_parser(subparsers, subcommand_name, subcommand_help, description, epilog=epilog)
